#!/usr/bin/env perl
# pbo-gen-makefile.pl - generate a makefile for specified pbo-cpp sources

# call as: pbo-gen-makefile.pl <pbo-cpp source file 1> <pbo-cpp source file 2> ...
# input: <pbo-source file>
# output: Makefile.pbo-example

# (C) Quinn Hsu, University of British Columbia 2012-02

# version 0.1
#	- initial port from the weaver

my($ext) = "\.pbo-cpp";

# save the names of the input filenames
my(@inputFiles) = ();

my($makefilename);

# while command line arguments have the right 
# file extension
for my $arg (@ARGV) {
	if ($arg =~ /^(.+$ext)$/) {
		push(@inputFiles, $1);
		unless (-e $1) {
			printf "ERROR: Specified PbO source $1 does not exist.  Please check input file list.\n";
			exit(-1);
		}
	}
	
	if ($arg =~ /^--rename$/) { $makefilename = "Makefile"; } 
	else { $makefilename = "Makefile.pbo-example"; }
}


my $filenames = join (" ", @inputFiles);

open (MAKEFILE, ">$makefilename");
print MAKEFILE <<"END";

.SUFFIXES: .pbo-cpp .c .o .cpp

# do not delete intermediate .c generated by weaver
.SECONDARY:
SOURCES	= $filenames
END

print MAKEFILE <<'END';
CSOURCES = $(SOURCES:.pbo-cpp=.cpp)
HIDDENS = $(addprefix ., $(CSOURCES))
CFILES	= $(HIDDENS) pbo_support.cpp
OFILES	= $(CFILES:.cpp=.o)

MAIN	= main
CC      = g++

all: ${MAIN}.exe

opt.o: opt.c opt.h
	${CC} -c opt.c

$(HIDDENS): pbo-weave-cpp.pl

$(CFILES): $(CSOURCES)

.pbo-cpp.cpp:
	perl pbo-weave-cpp.pl $(SOURCES)

.cpp.o:
	${CC} -o $@ -c $<

${MAIN}.exe: $(OFILES) opt.o
	${CC} -o ${MAIN}.exe  $(OFILES) opt.o

clean:
	rm -f .*.o $(CFILES) ${MAIN}.exe

END

close(MAKEFILE);